@@ -22,6 +22,7 @@ The following commands are currently supported:
     search
     stats
     stream
+    trends
 
 """
 
@@ -35,6 +36,7 @@ import shodan.helpers as helpers
 import threading
 import requests
 import time
+import json
 
 # The file converters that are used to go from .json.gz to various other formats
 from shodan.cli.converter import CsvConverter, KmlConverter, GeoJsonConverter, ExcelConverter, ImagesConverter
@@ -91,7 +93,7 @@ main.add_command(scan)
 
 @main.command()
 @click.option('--fields', help='List of properties to output.', default=None)
-@click.argument('input', metavar='<input file>')
+@click.argument('input', metavar='<input file>', type=click.Path(exists=True))
 @click.argument('format', metavar='<output format>', type=click.Choice(CONVERTERS.keys()))
 def convert(fields, input, format):
     """Convert the given input data file into a different format. The following file formats are supported:
@@ -100,6 +102,7 @@ def convert(fields, input, format):
 
     Example: shodan convert data.json.gz kml
     """
+    file_size = 0
     # Check that the converter allows a custom list of fields
     converter_class = CONVERTERS.get(format)
     if fields:
@@ -107,6 +110,9 @@ def convert(fields, input, format):
             raise click.ClickException('File format doesnt support custom list of fields')
         converter_class.fields = [item.strip() for item in fields.split(',')]  # Use the custom fields the user specified
 
+    # click.Path ensures that file path exists
+    file_size = os.path.getsize(input)
+
     # Get the basename for the input file
     basename = input.replace('.json.gz', '').replace('.json', '')
 
@@ -124,7 +130,7 @@ def convert(fields, input, format):
     # Initialize the file converter
     converter = converter_class(fout)
 
-    converter.process([input])
+    converter.process([input], file_size)
 
     finished_event.set()
     progress_bar_thread.join()
@@ -255,10 +261,11 @@ def count(query):
 
 
 @main.command()
+@click.option('--fields', help='Specify the list of properties to download instead of grabbing the full banner', default=None, type=str)
 @click.option('--limit', help='The number of results you want to download. -1 to download all the data possible.', default=1000, type=int)
 @click.argument('filename', metavar='<filename>')
 @click.argument('query', metavar='<search query>', nargs=-1)
-def download(limit, filename, query):
+def download(fields, limit, filename, query):
     """Download search results and save them in a compressed JSON file."""
     key = get_api_key()
 
@@ -277,6 +284,10 @@ def download(limit, filename, query):
     if not filename.endswith('.json.gz'):
         filename += '.json.gz'
 
+    # Strip out any whitespace in the fields and turn them into an array
+    if fields is not None:
+        fields = [item.strip() for item in fields.split(',')]
+
     # Perform the search
     api = shodan.Shodan(key)
 
@@ -287,10 +298,10 @@ def download(limit, filename, query):
         raise click.ClickException('The Shodan API is unresponsive at the moment, please try again later.')
 
     # Print some summary information about the download request
-    click.echo('Search query:\t\t\t%s' % query)
-    click.echo('Total number of results:\t%s' % total)
-    click.echo('Query credits left:\t\t%s' % info['unlocked_left'])
-    click.echo('Output file:\t\t\t%s' % filename)
+    click.echo('Search query:\t\t\t{}'.format(query))
+    click.echo('Total number of results:\t{}'.format(total))
+    click.echo('Query credits left:\t\t{}'.format(info['unlocked_left']))
+    click.echo('Output file:\t\t\t{}'.format(filename))
 
     if limit > total:
         limit = total
@@ -302,7 +313,7 @@ def download(limit, filename, query):
     with helpers.open_file(filename, 'w') as fout:
         count = 0
         try:
-            cursor = api.search_cursor(query, minify=False)
+            cursor = api.search_cursor(query, minify=False, fields=fields)
             with click.progressbar(cursor, length=limit) as bar:
                 for banner in bar:
                     helpers.write_banner(fout, banner)
@@ -485,7 +496,7 @@ def search(color, fields, limit, separator, query):
     # Perform the search
     api = shodan.Shodan(key)
     try:
-        results = api.search(query, limit=limit)
+        results = api.search(query, limit=limit, minify=False, fields=fields)
     except shodan.APIError as e:
         raise click.ClickException(e.value)
 
@@ -794,6 +805,102 @@ def stream(streamer, fields, separator, datadir, asn, alert, countries, custom_f
             stream = _create_stream(stream_type, stream_args, timeout=timeout)
 
 
+@main.command()
+@click.option('--facets', help='List of facets to get summary information on, if empty then show query total results over time', default='', type=str)
+@click.option('--filename', '-O', help='Save the full results in the given file (append if file exists).', default=None)
+@click.option('--save', '-S', help='Save the full results in the a file named after the query (append if file exists).', default=False, is_flag=True)
+@click.argument('query', metavar='<search query>', nargs=-1)
+def trends(filename, save, facets, query):
+    """Search Shodan historical database"""
+    key = get_api_key()
+    api = shodan.Shodan(key)
+
+    # Create the query string out of the provided tuple
+    query = ' '.join(query).strip()
+    facets = facets.strip()
+
+    # Make sure the user didn't supply an empty query or facets
+    if query == '':
+        raise click.ClickException('Empty search query')
+
+    # Convert comma-separated facets string to list
+    parsed_facets = []
+    for facet in facets.split(','):
+        if not facet:
+            continue
+
+        parts = facet.strip().split(":")
+        if len(parts) > 1:
+            parsed_facets.append((parts[0], parts[1]))
+        else:
+            parsed_facets.append((parts[0]))
+
+    # Perform the search
+    try:
+        results = api.trends.search(query, facets=parsed_facets)
+    except shodan.APIError as e:
+        raise click.ClickException(e.value)
+
+    # Error out if no results were found
+    if results['total'] == 0:
+        raise click.ClickException('No search results found')
+
+    result_facets = []
+    if results.get("facets"):
+        result_facets = list(results["facets"].keys())
+
+    # Save the results first to file if user request
+    if filename or save:
+        if not filename:
+            filename = '{}-trends.json.gz'.format(query.replace(' ', '-'))
+        elif not filename.endswith('.json.gz'):
+            filename += '.json.gz'
+
+        # Create/ append to the file
+        with helpers.open_file(filename) as fout:
+            for index, match in enumerate(results['matches']):
+                # Append facet info to make up a line
+                if result_facets:
+                    match["facets"] = {}
+                    for facet in result_facets:
+                        match["facets"][facet] = results['facets'][facet][index]['values']
+
+                line = json.dumps(match) + '\n'
+                fout.write(line.encode('utf-8'))
+
+        click.echo(click.style(u'Saved results into file {}'.format(filename), 'green'))
+
+    # We buffer the entire output so we can use click's pager functionality
+    output = u''
+
+    # Output examples:
+    # - Facet by os
+    # 2017-06
+    #   os
+    #     Linux 3.x                                                        146,502
+    #     Windows 7 or 8                                                     2,189
+    #
+    # - Without facets
+    # 2017-06               19,799,459
+    # 2017-07               21,077,099
+    if result_facets:
+        for index, match in enumerate(results['matches']):
+            output += click.style(match['month'] + u'\n', fg='green')
+            if match['count'] > 0:
+                for facet in result_facets:
+                    output += click.style(u'  {}\n'.format(facet), fg='cyan')
+                    for bucket in results['facets'][facet][index]['values']:
+                        output += u'    {:60}{}\n'.format(click.style(bucket['value'], bold=True), click.style(u'{:20,d}'.format(bucket['count']), fg='green'))
+            else:
+                output += u'{}\n'.format(click.style('N/A', bold=True))
+    else:
+        # Without facets, show query total results over time
+        for index, match in enumerate(results['matches']):
+            output += u'{:20}{}\n'.format(click.style(match['month'], bold=True), click.style(u'{:20,d}'.format(match['count']), fg='green'))
+
+    click.echo_via_pager(output)
+
+
 @main.command()
 @click.argument('ip', metavar='<IP address>')
 def honeyscore(ip):
