@@ -7,13 +7,15 @@ This module implements the Shodan API.
 
 :copyright: (c) 2014- by John Matherly
 """
+import math
+import os
 import time
 
 import requests
 import json
 
 from .exception import APIError
-from .helpers import api_request, create_facet_string
+from .helpers import create_facet_string
 from .stream import Stream
 
 
@@ -85,7 +87,7 @@ class Shodan:
 
         def __init__(self, parent):
             self.parent = parent
-        
+
         def create(self, provider, args, description=None):
             """Get the settings for the specified notifier that a user has configured.
 
@@ -101,9 +103,9 @@ class Shodan:
 
             if description:
                 args['description'] = description
-            
+
             return self.parent._request('/notifier', args, method='post')
-        
+
         def edit(self, nid, args):
             """Get the settings for the specified notifier that a user has configured.
 
@@ -114,7 +116,7 @@ class Shodan:
             :returns: dict -- fields are 'success' and 'id' of the notifier
             """
             return self.parent._request('/notifier/{}'.format(nid), args, method='put')
-        
+
         def get(self, nid):
             """Get the settings for the specified notifier that a user has configured.
 
@@ -137,7 +139,7 @@ class Shodan:
             :returns: A list of providers where each object describes a provider
             """
             return self.parent._request('/notifier/provider', {})
-        
+
         def remove(self, nid):
             """Delete the provided notifier.
 
@@ -253,6 +255,42 @@ class Shodan:
             """
             return self.parent._request('/org/member/{}'.format(user), {}, method='DELETE')['success']
 
+    class Trends:
+
+        def __init__(self, parent):
+            self.parent = parent
+
+        def search(self, query, facets):
+            """Search the Shodan historical database.
+
+            :param query: Search query; identical syntax to the website
+            :type query: str
+            :param facets: (optional) A list of properties to get summary information on
+            :type facets: str
+
+            :returns: A dictionary with 3 main items: matches, facets and total. Visit the website for more detailed information.
+            """
+            args = {
+                'query': query,
+                'facets': create_facet_string(facets),
+            }
+
+            return self.parent._request('/api/v1/search', args, service='trends')
+
+        def search_facets(self):
+            """This method returns a list of facets that can be used to get a breakdown of the top values for a property.
+
+            :returns: A list of strings where each is a facet name
+            """
+            return self.parent._request('/api/v1/search/facets', {}, service='trends')
+
+        def search_filters(self):
+            """This method returns a list of search filters that can be used in the search query.
+
+            :returns: A list of strings where each is a filter name
+            """
+            return self.parent._request('/api/v1/search/filters', {}, service='trends')
+
     def __init__(self, key, proxies=None):
         """Initializes the API object.
 
@@ -264,9 +302,11 @@ class Shodan:
         self.api_key = key
         self.base_url = 'https://api.shodan.io'
         self.base_exploits_url = 'https://exploits.shodan.io'
+        self.base_trends_url = 'https://trends.shodan.io'
         self.data = self.Data(self)
         self.dns = self.Dns(self)
         self.exploits = self.Exploits(self)
+        self.trends = self.Trends(self)
         self.labs = self.Labs(self)
         self.notifier = self.Notifier(self)
         self.org = self.Organization(self)
@@ -275,11 +315,15 @@ class Shodan:
         self._session = requests.Session()
         self.api_rate_limit = 1  # Requests per second
         self._api_query_time = None
+
         if proxies:
             self._session.proxies.update(proxies)
             self._session.trust_env = False
+        
+        if os.environ.get('SHODAN_API_URL'):
+            self.base_url = os.environ.get('SHODAN_API_URL')
 
-    def _request(self, function, params, service='shodan', method='get'):
+    def _request(self, function, params, service='shodan', method='get', json_data=None):
         """General-purpose function to create web requests to SHODAN.
 
         Arguments:
@@ -297,6 +341,7 @@ class Shodan:
         base_url = {
             'shodan': self.base_url,
             'exploits': self.base_exploits_url,
+            'trends': self.base_trends_url,
         }.get(service, 'shodan')
 
         # Wait for API rate limit
@@ -308,7 +353,13 @@ class Shodan:
         try:
             method = method.lower()
             if method == 'post':
-                data = self._session.post(base_url + function, params)
+                if json_data:
+                    data = self._session.post(base_url + function, params=params,
+                                            data=json.dumps(json_data),
+                                            headers={'content-type': 'application/json'},
+                        )
+                else:
+                    data = self._session.post(base_url + function, params)
             elif method == 'put':
                 data = self._session.put(base_url + function, params=params)
             elif method == 'delete':
@@ -480,7 +531,7 @@ class Shodan:
         """
         return self._request('/shodan/scan/{}'.format(scan_id), {})
 
-    def search(self, query, page=1, limit=None, offset=None, facets=None, minify=True):
+    def search(self, query, page=1, limit=None, offset=None, facets=None, minify=True, fields=None):
         """Search the SHODAN database.
 
         :param query: Search query; identical syntax to the website
@@ -495,6 +546,8 @@ class Shodan:
         :type facets: str
         :param minify: (optional) Whether to minify the banner and only return the important data
         :type minify: bool
+        :param fields: (optional) List of properties that should get returned. This option is mutually exclusive with the "minify" parameter
+        :type fields: str
 
         :returns: A dictionary with 2 main items: matches and total. If facets have been provided then another property called "facets" will be available at the top-level of the dictionary. Visit the website for more detailed information.
         """
@@ -512,9 +565,12 @@ class Shodan:
         if facets:
             args['facets'] = create_facet_string(facets)
 
+        if fields and isinstance(fields, list):
+            args['fields'] = ','.join(fields)
+
         return self._request('/shodan/host/search', args)
 
-    def search_cursor(self, query, minify=True, retries=5):
+    def search_cursor(self, query, minify=True, retries=5, fields=None):
         """Search the SHODAN database.
 
         This method returns an iterator that can directly be in a loop. Use it when you want to loop over
@@ -532,17 +588,25 @@ class Shodan:
         :returns: A search cursor that can be used as an iterator/ generator.
         """
         page = 1
+        total_pages = 0
         tries = 0
 
-        # Placeholder results object to make the while loop below easier
-        results = {
-            'matches': [True],
-            'total': None,
-        }
-
-        while results['matches']:
+        # Grab the initial page and use the total to calculate the expected number of pages
+        results = self.search(query, minify=minify, page=page, fields=fields)
+        if results['total']:
+            total_pages = int(math.ceil(results['total'] / 100))
+                    
+        for banner in results['matches']:
             try:
-                results = self.search(query, minify=minify, page=page)
+                yield banner
+            except GeneratorExit:
+                return  # exit out of the function
+        page += 1
+
+        # Keep iterating over the results from page 2 onwards
+        while page <= total_pages:
+            try:
+                results = self.search(query, minify=minify, page=page, fields=fields)
                 for banner in results['matches']:
                     try:
                         yield banner
@@ -658,8 +722,7 @@ class Shodan:
             'expires': expires,
         }
 
-        response = api_request(self.api_key, '/shodan/alert', data=data, params={}, method='post',
-                               proxies=self._session.proxies)
+        response = self._request('/shodan/alert', params={}, json_data=data, method='post')
 
         return response
 
@@ -679,8 +742,7 @@ class Shodan:
             },
         }
 
-        response = api_request(self.api_key, '/shodan/alert/{}'.format(aid), data=data, params={}, method='post',
-                               proxies=self._session.proxies)
+        response = self._request('/shodan/alert/{}'.format(aid), params={}, json_data=data, method='post')
 
         return response
 
@@ -691,9 +753,9 @@ class Shodan:
         else:
             func = '/shodan/alert/info'
 
-        response = api_request(self.api_key, func, params={
+        response = self._request(func, params={
             'include_expired': include_expired,
-        }, proxies=self._session.proxies)
+        })
 
         return response
 
@@ -701,8 +763,7 @@ class Shodan:
         """Delete the alert with the given ID."""
         func = '/shodan/alert/{}'.format(aid)
 
-        response = api_request(self.api_key, func, params={}, method='delete',
-                               proxies=self._session.proxies)
+        response = self._request(func, params={}, method='delete')
 
         return response
 
@@ -728,17 +789,17 @@ class Shodan:
         # a different API endpoint.
         if trigger in ('vulnerable', 'vulnerable_unverified') and vulns and isinstance(vulns, list):
             return self._request('/shodan/alert/{}/trigger/{}/ignore/{}:{}/{}'.format(aid, trigger, ip, port, ','.join(vulns)), {}, method='put')
-        
+
         return self._request('/shodan/alert/{}/trigger/{}/ignore/{}:{}'.format(aid, trigger, ip, port), {}, method='put')
 
     def unignore_alert_trigger_notification(self, aid, trigger, ip, port):
         """Re-enable trigger notifications for the provided IP and port"""
         return self._request('/shodan/alert/{}/trigger/{}/ignore/{}:{}'.format(aid, trigger, ip, port), {}, method='delete')
-    
+
     def add_alert_notifier(self, aid, nid):
         """Enable the given notifier for an alert that has triggers enabled."""
         return self._request('/shodan/alert/{}/notifier/{}'.format(aid, nid), {}, method='put')
-    
+
     def remove_alert_notifier(self, aid, nid):
         """Remove the given notifier for an alert that has triggers enabled."""
         return self._request('/shodan/alert/{}/notifier/{}'.format(aid, nid), {}, method='delete')
